//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011"
#property link      ""

#include <CustPrevCalculated.mqh>

//+------------------------------------------------------------------+
//| Result of working of the function of calculation                 |
//+------------------------------------------------------------------+
enum CPCATRResultCode
  {
   CPCATRRC_NotInitialized,             // Class is not initialized
   CPCATRRC_NoData,                     // Not enough data
   CPCATRRC_NotChanged,                 // There has been no changes
   CPCATRRC_Changed                     // There has been changes
  };

//+------------------------------------------------------------------+
//| Structure of one bar of the indicator                            |
//+------------------------------------------------------------------+
struct ATRBar
  {
   double ATR;
   double TR;
  };

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
class CCustATR
  {
protected:
   bool Initialized;                         // The variable for protection from a programming error of class initialization
   bool UsesExternalData;                    // Use external data
   // Main parameters
   //----------------------------
   int BarsLimit;                            // Limit of bars for calculation is set once during initialization of the class
   uchar iAtrPeriod;
   // Arrays for data storage
   //----------------------------
   uint DataBarsCount;                       // The number of significant bars for that are stored in the array for calculation, cannot exceed BarsLimit
   uint DataBarsCalculated;                  // The number of bars that are not changed for sure
   uint DataStartInd;                        // Initial value of indexing in the array for calculation (for wraparound addressing)
   ATRBar Buf[];                           // The array of data for the calculation itself
   uint ATRDataBarsCalculated;             // The number of calculated bars of the indicator
   //----------------------------
   bool ATRDataBarsCuttingLeft;            // Set only to 'true' at each call of the PrepareData() procedure
   CCustPrevCalculated *CustPrevCalculated;
   // Auxiliary variables
   //----------------------------
   int __Digits;
   double __Point;
   uint PInd(uint I, bool AsSeries) {return( AsSeries?((DataStartInd+DataBarsCount-1-I) % BarsLimit):((DataStartInd+I) % BarsLimit) );}
public:
   // Constructor and destructor
   void CCustATR() {Initialized = false;}
   void ~CCustATR() {if (CustPrevCalculated != NULL) delete CustPrevCalculated;}
   bool ItIsInitialized() {return(Initialized);}
   // Manual initialization of the class
   bool Init(CCustPrevCalculated *CPC, string Instr, ENUM_TIMEFRAMES TF, int Limit, CPCHistorySynchMode HSM, uchar HS, uint HSMinute, uchar AtrPeriod);
   // Main procedure of synchronization of data
   CPCPrepareDataResultCode PrepareData(CPCPrepareDataResultCode resData);   // Procedure for loading new data for the calculation
   // Main procedure of calculation of the indicator
   CPCATRResultCode Calculate();
   // The function of access to the calculated data
   uint GetBarsCount() {return(DataBarsCount);}                   // Returns the number of available bars
   uint GetBarsCalculated() {return(ATRDataBarsCalculated);}      // Returns the number of calculated bars
   double GetATR(uint shift, bool AsSeries);                      // Returns the value of the ATR buffer for a bar
  };
//+------------------------------------------------------------------+
bool CCustATR::Init(CCustPrevCalculated *CPC, string Instr, ENUM_TIMEFRAMES TF, int Limit, CPCHistorySynchMode HSM, uchar HS, uint HSMinute, uchar AtrPeriod)
{
   if (Initialized)
      return(false);
   
   if (CPC == NULL)
     {
      CustPrevCalculated = new CCustPrevCalculated;
      UsesExternalData = false;
     } else {
      CustPrevCalculated = CPC;
      UsesExternalData = true;
     }
   if (CustPrevCalculated.InitData(Instr, TF, Limit, HSM, HS, HSMinute))
     {
      if (ArrayResize(Buf, Limit) == -1)
         return(Initialized);

      BarsLimit = Limit;
      iAtrPeriod = AtrPeriod;
      
      __Digits = (int)SymbolInfoInteger(Instr, SYMBOL_DIGITS);
      __Point = SymbolInfoDouble(Instr, SYMBOL_POINT);
      
      Initialized = true;
      return(true);
     } else {
      Initialized = false;
      return(false);
     }
}
//+------------------------------------------------------------------+
CPCPrepareDataResultCode CCustATR::PrepareData(CPCPrepareDataResultCode resData)
{
   if (!Initialized)
      return(CPCPDRC_NoData);
   
   // Prepare trade data for working
   CPCPrepareDataResultCode res = resData;
   DataBarsCount = CustPrevCalculated.GetDataBarsCount();
   DataBarsCalculated = CustPrevCalculated.GetDataBarsCalculated();
   DataStartInd = CustPrevCalculated.GetDataStartInd();
   
   // If the bars are cut from the left side, set the flag in the auxiliary variable
   // Only 'true' value is set, 'false' is zeroized on its own if necessary
   if (CustPrevCalculated.GetDataBarsCuttingLeft())
      ATRDataBarsCuttingLeft = true;
   
   // Zeroize the newly added or synchronized bars in the buffer
   for (uint i = (DataBarsCalculated == 0)?0:(DataBarsCalculated+1); i < DataBarsCount; i++)
     {
      Buf[PInd(i, false)].ATR = EMPTY_VALUE;
      Buf[PInd(i, false)].TR = EMPTY_VALUE;
     }
   
   // If there is a rollback in history, move the calculated point
   ATRDataBarsCalculated = MathMin(DataBarsCalculated, ATRDataBarsCalculated);
   
   return(res);
}
//+------------------------------------------------------------------+
double CCustATR::GetATR(uint shift, bool AsSeries)
{
   if ( shift > (DataBarsCount-1) )
      return(EMPTY_VALUE);
   return(Buf[PInd(shift, AsSeries)].ATR);
}
//+------------------------------------------------------------------+
CPCATRResultCode CCustATR::Calculate()
{
   if (!Initialized)
      return(CPCATRRC_NotInitialized);
   
   // If the data provider is not external, synchronize the history
   if (!UsesExternalData)
     {
      CPCPrepareDataResultCode resCPCPD = CustPrevCalculated.PrepareData();
      CPCPrepareDataResultCode resPD = PrepareData(resCPCPD);

      if (resPD == CPCPDRC_NoData)
         return(CPCATRRC_NoData);
      if (resPD == CPCPDRC_NoRecountNotRequired)
         return(CPCATRRC_NotChanged);
     }
   
   // Check if there are enough bar for calculation
   if (DataBarsCount <= iAtrPeriod)
      return(CPCATRRC_NoData);
   
   uint BarsForRecalculation;
   if ( DataBarsCalculated != 0 )
      BarsForRecalculation = DataBarsCount - ATRDataBarsCalculated - 1;
   else
     {
      Buf[PInd(0, false)].TR = 0.0;
      Buf[PInd(0, false)].ATR = 0.0;
      //--- filling out the array of True Range values for each period
      for (uint i = 1; i < DataBarsCount; i++)
         Buf[PInd(i, false)].TR = MathMax(CustPrevCalculated.GetDataHigh(i, false), CustPrevCalculated.GetDataClose(i-1, false)) - 
                                  MathMin(CustPrevCalculated.GetDataLow(i, false), CustPrevCalculated.GetDataClose(i-1, false));
      //--- first AtrPeriod values of the indicator are not calculated
      double firstValue = 0.0;
      for (uint i = 1; i <= iAtrPeriod; i++)
        {
         Buf[PInd(i, false)].ATR = 0;
         firstValue += Buf[PInd(i, false)].TR;
        }
      //--- calculating the first value of the indicator
      firstValue /= iAtrPeriod;
      Buf[PInd(iAtrPeriod, false)].ATR = firstValue;
      
      BarsForRecalculation = DataBarsCount - iAtrPeriod - 2;
     }

   // The history was cut from the left side
   if (ATRDataBarsCuttingLeft)
     {
      ATRDataBarsCuttingLeft = false;
     }

   //======================================================
   //======== main loop ===============================
   //======================================================
   CPCATRResultCode res = CPCATRRC_NotChanged;
   
   for (uint i = (DataBarsCount - BarsForRecalculation - 1); i < DataBarsCount; i++)
     {
      Buf[PInd(i, false)].TR = MathMax(CustPrevCalculated.GetDataHigh(i, false), CustPrevCalculated.GetDataClose(i-1, false)) - 
                               MathMin(CustPrevCalculated.GetDataLow(i, false), CustPrevCalculated.GetDataClose(i-1, false));
      Buf[PInd(i, false)].ATR = Buf[PInd(i-1, false)].ATR + (Buf[PInd(i, false)].TR-Buf[PInd(i-iAtrPeriod, false)].TR) / iAtrPeriod;
     
      res = CPCATRRC_Changed;
      ATRDataBarsCalculated = i + 1; // right here, because we can exit earlier
     } // for (main)
   //----------
   return(res);
}